// Time.h: interface for the TTime class and other time-related functions.
//

#if !defined(_TTIME_H_)
#define _TTIME_H_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#undef DECL
#if defined(_AFXDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif

///////////////////////////////////////////////////////////////////////////
// TTimeSpan class
//
class DECL TTimeSpan
{
   // TTimeSpan represents a relative time span. The span is stored with 
   // a resolution of 1ms.

   // 
   // Interface
   // 
   public:
   
   // Constructors and destructor.
   TTimeSpan() {ticks = 0;};
   TTimeSpan(const TTimeSpan& ts) {ticks = ts.ticks;};
   TTimeSpan(INT64 t) {ticks = t;};
   TTimeSpan(int days, int hours, int mins, int secs, int ms = 0)
   {
      ticks = ((INT64) days * 86400000) + ((INT64) hours * 3600000) + ((INT64) mins * 60000) 
         + ((INT64) secs * 1000) + ((INT64) ms);
   };

   // Accessors for days, hours, totals, etc.
   int GetDays() const {return (int)(ticks / 86400000);};
   int GetHours() const {return (int)(GetTotalHours() - GetDays() * 24);};
   int GetMinutes() const {return (int)(GetTotalMinutes() - (INT64)GetTotalHours() * 60);};
   int GetSeconds() const {return (int)(GetTotalSeconds() - GetTotalMinutes() * 60);};
   int GetMilliseconds() const {return (int)(GetTotalMilliseconds() - GetTotalSeconds() * 1000);};
   int GetTotalHours() const {return (int)(ticks / 3600000);};
   INT64 GetTotalMinutes() const {return ticks / 60000;};
   INT64 GetTotalSeconds() const {return ticks / 1000;};
   INT64 GetTotalMilliseconds() const {return ticks;};
   
   // Returns ticks (milliseconds).
   INT64 GetTicks() {return ticks;};

   // Returns a CTimeSpan object equivalent to the current object.
   CTimeSpan GetCTimeSpan() {return CTimeSpan((time_t)(ticks / 1000));};
   
   // Operators.
   TTimeSpan operator + (const TTimeSpan& ts) const {return TTimeSpan(ticks + ts.ticks);};
   TTimeSpan operator + (INT64 t) const {return TTimeSpan(ticks + t);};
   TTimeSpan operator - (const TTimeSpan& ts) const {return TTimeSpan(ticks - ts.ticks);};
   TTimeSpan operator - (INT64 t) const {return TTimeSpan(ticks - t);};
   TTimeSpan& operator += (const TTimeSpan& ts) {ticks += ts.ticks; return *this;};
   TTimeSpan& operator += (INT64 t) {ticks += t; return *this;};
   TTimeSpan& operator -= (const TTimeSpan& ts) {ticks -= ts.ticks; return *this;};
   TTimeSpan& operator -= (INT64 t) {ticks -= t; return *this;};
   TTimeSpan& operator = (const TTimeSpan& ts) {ticks = ts.ticks; return *this;};
   TTimeSpan& operator = (INT64 t) {ticks = t; return *this;};
   bool operator == (const TTimeSpan& ts) const {return ticks == ts.ticks;};
   bool operator != (const TTimeSpan& ts) const {return ticks != ts.ticks;};
   bool operator < (const TTimeSpan& ts) const {return ticks < ts.ticks;};
   bool operator < (INT64 i) const {return ticks < i;};
   bool operator > (const TTimeSpan& ts) const {return ticks > ts.ticks;};
   bool operator > (INT64 i) const {return ticks > i;};
   bool operator <= (const TTimeSpan& ts) const {return ticks <= ts.ticks;};
   bool operator <= (INT64 i) const {return ticks <= i;};
   bool operator >= (const TTimeSpan& ts) const {return ticks >= ts.ticks;};
   bool operator >= (INT64 i) const {return ticks >= i;};

   // Return the number of ticks per second.
   static INT64 GetTicksPerSecond() {return 1000;};

   // Format time string
   CString FormatTime();
   
   // Method to test the class. Must be run in debug build. 
   #ifdef _DEBUG
   static void Test();
   #endif
   
   // 
   // Implementation
   // 
   protected:
   friend class TTime;

   // Attributes
   INT64 ticks;
};


///////////////////////////////////////////////////////////////////////////
// TTime class
//
class DECL TTime
{
   // TTime represents an absolute time. The time is stored as 1ms intervals 
   // ("ticks") since January 1, 1601. The units are compatible with TTimeSpan. 
   // Reference is UTC; there is no provision to account for time-zones or 
   // daylight savings time. 

   // 
   // Interface
   // 
   public:
   
   // Constructors and destructor.
   TTime() {ticks = 0;};
   TTime(const TTime& t) {ticks = t.ticks;};
   TTime(INT64 t) {ticks = t;};
   TTime(const FILETIME& ft);
   TTime(const SYSTEMTIME& sysTime);
   TTime(int year, int month, int day = 1, int hour = 0, int min = 0, 
      int sec = 0, int ms = 0);

   // Returns current time.
   #undef GetCurrentTime
   static TTime GetCurrentTime();
   
   // Accessors for year, month, etc. These are zero-based.
   int GetYear() const;
   int GetMonth() const;
   int GetDay() const;
   int GetHour() const;
   int GetMinute() const;
   int GetSecond() const;
   int GetMillisecond() const;

   // Leap year?
   bool IsLeapYear() const;
   
   // Day of the year returns n'th day of the year (e.g., 1/1 = 1, 1/2 = 2, ...)
   int GetDayOfYear() const;
   
   // Days in the current month (accounts for leap year)
   int GetDaysInMonth() const;
   
   // Returns the day of the week (1 = Sunday, 2 = Monday, ..., 7 = Saturday).
   int GetDayOfWeek() const;

   // Returns seconds into the day for this object.
   int GetSecondsIntoDay();
   
   // Returns the value of this instance in 1ms intervals since January 1, 1601.
   INT64 GetTicks() {return ticks;};
   
   // Stores the value of this instance in a SYSTEMTIME object.
   BOOL GetAsSystemTime(SYSTEMTIME& sysTime) const;

   // Returns a CTime object initialized using the time in the current object.
   CTime GetCTime() {return CTime(GetYear(), GetMonth(), GetDay(), GetHour(), 
         GetMinute(), GetSecond());};
   
   // Operators.
   TTime operator + (TTimeSpan ts) const {return TTime(ticks + ts.ticks);};
   TTime operator + (INT64 t) const {return TTime(ticks + t);};
   TTime operator - (const TTimeSpan& ts) const {return TTime(ticks - ts.ticks);};
   TTime operator - (INT64 t) const {return TTime(ticks - t);};
   TTimeSpan operator - (const TTime& t) const {return TTimeSpan(ticks - t.ticks);};
   TTime& operator += (const TTimeSpan& ts) {ticks += ts.ticks; return *this;};
   TTime& operator += (INT64 t) {ticks += t; return *this;};
   TTime& operator -= (const TTimeSpan& ts) {ticks -= ts.ticks; return *this;};
   TTime& operator -= (INT64 t) {ticks -= t; return *this;};
   TTime& operator = (const TTime& t) {ticks = t.ticks; return *this;};
   TTime& operator = (INT64 t) {ticks = t; return *this;};
   bool operator == (const TTime& t) const {return ticks == t.ticks;};
   bool operator != (const TTime& t) const {return ticks != t.ticks;};
   bool operator < (const TTime& t) const {return ticks < t.ticks;};
   bool operator > (const TTime& t) const {return ticks > t.ticks;};
   bool operator <= (const TTime& t) const {return ticks <= t.ticks;};
   bool operator >= (const TTime& t) const {return ticks >= t.ticks;};

   // Returns current time in performance counter intervals since January 1, 1601.
   // This is the same as the kernel's GetRTCTicksExact, but is easier to call 
   // since no out arg is required. 
   static INT64 GetCurrentTimePC();

   // Returns the number of milliseconds since the last system boot (similar to 
   // GetTickCount(), but with 64bit resolution). 
   static INT64 GetTicksSinceBoot();

   // Return the number of ticks per second.
   static INT64 GetTicksPerSecond() {return 1000;};

   // Format time into string.
   CString FormatTimeDate();
   CString FormatDateTime();
   CString FormatTime();
   CString FormatDate();
   
   // Method to test the class. Must be run in debug build. 
   #ifdef _DEBUG
   static void Test();
   #endif
   
   // 
   // Implementation
   // 
   protected:
   friend class TTimeSpan;

   // Attributes
   INT64 ticks;
};


///////////////////////////////////////////////////////////////////////////
// Exported time-related functions
//
bool DECL ValidateDateTime(CString& strDateTime);
bool DECL ValidateDate(CString& strDate);
bool DECL ValidateTime(const CString& strTime, bool bMSec = false, bool m_b23HrLimit = false);
bool DECL ValidateInt(const CString& strInt, int iMin = INT_MIN, int iMax = INT_MAX);
DWORD DECL StrToMS(const CString& string);
TTimeSpan DECL StrToTimeSpan(const CString& string);
TTime DECL StrToTime(const CString& string);
TTime DECL NextScheduledTime(TTime TheTime, TTimeSpan Interval, TTimeSpan Offset);
DWORD DECL NextScheduledTime(DWORD TheTime, DWORD Interval, DWORD Offset);
DWORD DECL MSTil(const TTime& tNext);
DWORD DECL WaitForTime(TTime t, HANDLE hEvent);
DWORD DECL WaitForTime(TTime t, HANDLE hEvents[], int nCount);

#endif // !defined_TTIME_H_)
